#include <iostream>
using std::ostream;
using std::istream;

#include "Matrix.h"

//      
void Matrix:: swap(Row &a, Row &b)
{
    Row temp=a;
    a=b;
    b=temp;
}

//     
void Matrix:: resize(int r,int c,double fill)
{
    if(row!=r || column!=c)
    {
        row=r;
        column=c;
        delete [] matrix;
        matrix=new Row[row];
    }

    for (int i=0;i<row;i++)
     matrix[i].resize(column,fill);
}


//
Matrix::Matrix(int r,int c,double fill)
{
    row=(r>0 ? r : 1);
    column=(c>0 ? c : 1);
    matrix= new Row[row];
    //  
    for (int i=0;i<row;i++)
     matrix[i].resize(column,fill);
}


// 
Matrix::Matrix(const Matrix &right)
{
    if (*this!=right) //  
    {
        //   
        if (row!=right.row || column!=right.column)
        {
            delete [] matrix;
            row=right.row;
            column=right.column;
            matrix=new Row[row];

        }
        for (int i=0;i<row;i++)
         matrix[i]=right.matrix[i];
    }
}


Matrix::~Matrix()
{
    delete [] matrix;
}


//    
Row& Matrix:: operator[](int numb)
{
    //     
    if(numb<=row)
     return matrix[numb];
    //   -
    //else
    // return (Row a());
}

const Row& Matrix:: operator[](int numb) const
{
    //     
    if(numb<=row)
     return matrix[numb];
    //   -
    //else
     //return (new Row());
}

//  
Matrix& Matrix:: operator+(Matrix &right)
{
    Matrix temp(*this);

    //     
    //     
    if (row==right.row && column==right.column)
    {
        for (int i=0;i<row;i++)
         for (int j=0;j<column;j++)
         temp.matrix[i][j]+=right.matrix[i][j];
    }

    //  
    else
     temp.resize();
    return temp;
}

Matrix& Matrix:: operator+=(Matrix &right)
{
    *this=*this+right;
    return *this;
}

Matrix Matrix:: operator*(Matrix &right)
{
    //   
    //    
    //   
    Matrix temp(row,right.column);

    //   
    if (column==right.row)
    {
        for (int i = 0; i < row; i++)
         for (int j = 0; j < right.column; j++)
          for (int k = 0; k < column; k++)
           temp.matrix[i][j] += temp[i][k]*right.matrix[k][j];
    }
    else
     temp.resize();
    return temp;
}

Matrix& Matrix:: operator*=(Matrix &right)
{
    *this=*this*right;
    return *this;
}

Matrix& operator*(Matrix &left,const double right)
{
    //
    Matrix temp(left);

    for (int i=0;i<left.row;i++)
     for (int j=0;j<left.column;j++)
      temp.matrix[i][j]*=right;

    return temp;
}

Matrix& operator*=(Matrix &left, const double right)
{
    left=left*right;
    return left;
}

Matrix operator*(const double left, const Matrix &right)
{
    return (right * left);
}

Matrix& Matrix:: operator-(Matrix &right)
{
    return (*this +right*(-1.0));
}

Matrix& Matrix:: operator-=(Matrix &right)
{
    return (*this=*this + right*(-1.0));
}

Matrix Matrix:: operator/(Matrix &right)
{
    return *this * right.inverted();
}

Matrix Matrix:: operator/=(Matrix &right)
{
    *this=*this * right.inverted();
    return *this;
}

bool Matrix:: operator==(const Matrix &right)
{
    //      
    // false
    if(row!=right.row || column!=right.column)
     return false;

    //   
    // false
    for (int i=0;i<row;i++)
     if(matrix[i]!=right.matrix[i])
      return false;
    //   
    //  true
    return true;
}

bool Matrix:: operator!=(const Matrix &right)
{
    return !(*this==right);
}

Matrix& Matrix:: operator=(const Matrix &right)
{
    //  
    if(*this!=right)
    {
        //    
        if (row!=right.row || column!=right.column)
        {
            row=right.row;
            column=right.column;
            delete [] matrix;
            matrix=new Row[row];

            //  
            //    
            for (int i=0;i<row;i++)
             matrix[i].resize(column);
        }

        //   
        // 
        for (int i=0;i<row;i++)
         matrix[i]=right.matrix[i];
    }
    return *this;
}

double Matrix:: det()
{
    Matrix temp(*this);
    double result = 1.0;// 
    int dim = temp.row;
    int sign = 1; // 
    bool key = true;

    //   
    if (temp.row == temp.column)
    {
        for (int k=0; k<dim;k++)
        {
            if (temp.matrix[k][k]==0.0)
            {
                key=false;
                for (int i=k+1;i<dim;i++)
                {
                    if (temp.matrix[i][k]!=0.0)
                    {
                        key=true;
                        swap(temp.matrix[k], temp.matrix[i]);
                        sign*=-1;
                        break;
                    }
                }
            }
            if (!key)
             return 0.0;

            for (int i=k+1;i<dim;i++)
            {
                double multi = temp.matrix[i][k]/temp.matrix[k][k];
                for (int j = 0; j < dim; ++j)
                 temp.matrix[i][j]-=multi*temp.matrix[k][j];
            }
            result*=temp.matrix[k][k];
        }
    }
    else
     return 0.0;
    return sign*result;
}

Matrix Matrix:: inverted()
{
    //  
    Matrix temp(row, row);
    //   
    Matrix copy(*this);
    int dim = row;
    if (det() != 0.0)
    {
        for (int i= 0; i<dim; i++)
            temp.matrix[i][i] = 1.0;
        for (int k=0;k<dim;k++)
        {
            if (copy.matrix[k][k]==0.0)
            {
                for (int i=k+1;i<dim;i++)
                {
                    if (copy.matrix[i][k]!=0.0)
                    {
                        swap(copy.matrix[k], copy.matrix[i]);
                        swap(temp.matrix[k], temp.matrix[i]);

                        break;
                    }
                }
            }

            double div = copy.matrix[k][k];
            for (int j=0;j<dim;j++)
            {
                copy.matrix[k][j] /= div;
                temp.matrix[k][j] /= div;
            }

            for (int i=k+1;i<dim;i++)
            {
                double multi=copy.matrix[i][k];
                for (int j=0;j<dim;j++)
                {
                    copy.matrix[i][j] -= multi * copy.matrix[k][j];
                    temp.matrix[i][j] -= multi* temp.matrix[k][j];
                }
            }
        }
        for (int k=dim-1;k>0;k--)
        {
            for (int i=k-1;i+1>0;i--)
            {
                double multi = copy.matrix[i][k];

                for (int j=0;j<dim;j++)
                {
                    copy.matrix[i][j] -= multi * copy.matrix[k][j];
                    temp.matrix[i][j] -= multi * temp.matrix[k][j];
                }
            }
        }
    }
    else
     temp.resize();
    return temp;
}
